#!/usr/bin/env python
# funguy.py

import os
import logging
import importlib
import simplematrixbotlib as botlib
from dotenv import load_dotenv
import time
import sys
from plugins.config import FunguyConfig

# Setup logging
logging.basicConfig(format='%(asctime)s - %(levelname)s - %(message)s', level=logging.INFO)

# Load plugins (defined before plugin load/reload functions)
PLUGINS_DIR = "plugins"
PLUGINS = {}

def load_plugins():
    for plugin_file in os.listdir(PLUGINS_DIR):
        if plugin_file.endswith(".py"):
            plugin_name = os.path.splitext(plugin_file)[0]
            try:
                module = importlib.import_module(f"{PLUGINS_DIR}.{plugin_name}")
                PLUGINS[plugin_name] = module
                logging.info(f"Loaded plugin: {plugin_name}")
            except Exception as e:
                logging.error(f"Error loading plugin {plugin_name}: {e}")

def reload_plugins():
    global PLUGINS
    PLUGINS = {}
    # Unload modules from sys.modules
    for plugin_name in list(sys.modules.keys()):
        if plugin_name.startswith(PLUGINS_DIR + "."):
            del sys.modules[plugin_name]
    load_plugins()

def rehash_config(config):
    del config
    config = FunguyConfig()


# Load environment variables from .env file
load_dotenv()

# Load plugins
load_plugins()

# Bot configuration settings
MATRIX_URL = os.getenv("MATRIX_URL")
MATRIX_USER = os.getenv("MATRIX_USER")
MATRIX_PASS = os.getenv("MATRIX_PASS")

# Get credentials from env
creds = botlib.Creds(MATRIX_URL, MATRIX_USER, MATRIX_PASS)

# Bot configuration
config = FunguyConfig()
bot = botlib.Bot(creds, config)

@bot.listener.on_message_event
async def handle_commands(room, message):
    match = botlib.MessageMatch(room, message, bot, config.prefix)
    if match.is_not_from_this_bot() and match.prefix() and match.command("reload"):
        if str(message.sender) == config.admin_user:
            reload_plugins()
            await bot.api.send_text_message(room.room_id, "Plugins reloaded successfully")
        else:
            await bot.api.send_text_message(room.room_id, "You are not authorized to reload plugins.")

    for plugin_name, plugin_module in PLUGINS.items():
        await plugin_module.handle_command(room, message, bot, config.prefix, config)


    if match.is_not_from_this_bot() and match.prefix() and match.command("rehash"):
        if str(message.sender) == config.admin_user:
            rehash_config(config)
            await bot.api.send_text_message(room.room_id, "Config rehashed")
        else:
            await bot.api.send_text_message(room.room_id, "You are not authorized to reload plugins.")




bot.run()
